/* -*- mode: c; c-basic-offset: 4; -*- */
/* vi:set cindent ts=8 sts=4 sw=4: */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ncurses.h>
#include <unistd.h>

#include "main.h"

#include "curs_lib.h"
#include "lib.h"
#include "misc.h"
#include "options.h"
#include "play.h"
#include "playlist.h"

void play_song(int index)
{
    char cmd[LONG_STRING];
    char player[LONG_STRING];
    struct playlist_entry *p = first_entry;
    int i = 0;

    while (i++ < index)
	p = p->next;

    if (!has_registered_filename_extension(p->song)) {
	MSG_FILENAME_EXTENSION_NOT_REGISTERED(p->song);
	return;
    }
    if (extract_player_for_file(p->song, player, sizeof(player)) == NULL) {
	msg("Player for %s not found in the %s variable.", p->song, option_name(OPT_FILENAME_EXTENSIONS));
	return;
    }

    clear();
    refresh();
    snprintf(cmd, sizeof(cmd), "%s \"%s\"", player, p->path);
    endwin();			/* leave curses */
    system(cmd);
    refresh();			/* enter curses again */
}

void play_toggled_songs(bool continuous, bool random)
{
    struct playlist_entry *p = NULL;
    int ch = 0;
    bool prompt = FALSE;
    char cmd[LONG_STRING];
    char player[LONG_STRING];

    if (random) {

	/* play every toggled song in the playlist once in random order */
	/* before starting over again (if continuous is true) */

	int *toggled_songs_array;
	bool *played_songs_array;
	int j = 0, k = 0, l, toggled_songs = 0;
	int i;

	for (p = first_entry; p; p = p->next) {
	    if (p->toggled) {

		if (!has_registered_filename_extension(p->song))
		    MSG_FILENAME_EXTENSION_NOT_REGISTERED(p->song);
		toggled_songs++;
	    }
	}
	toggled_songs_array = safe_malloc(toggled_songs * sizeof(*toggled_songs_array));
	for (p = first_entry; p; p = p->next) {
	    if (p->toggled) {
		toggled_songs_array[k] = j;
		k++;
	    }
	    j++;
	}

	played_songs_array = safe_malloc(toggled_songs * sizeof(*played_songs_array));


    while_loop_again:
	/* set all elements to `FALSE' */
	memset(played_songs_array, FALSE, toggled_songs * sizeof(*played_songs_array));
	l = 0;
	while (l < toggled_songs) {

	give_me_another_number:
	    i = rand() % toggled_songs;

	    if (i == toggled_songs)	/* make sure we don't go beyond the bounds */
		i--;
	    /* i is now the index into the two dynamic arrays */

	    if (played_songs_array[i] == TRUE)
		goto give_me_another_number;

	    played_songs_array[i] = TRUE;
	    j = toggled_songs_array[i];		/* j is now the index in playlist */
	    l++;

	    p = first_entry;
	    k = 0;
	    while (k++ < j)
		p = p->next;

	    if (extract_player_for_file(p->song, player, sizeof(player)) == NULL) {
		clear();
		msg("Player for %s not found in the %s variable.", p->song, option_name(OPT_FILENAME_EXTENSIONS));
		safe_free(toggled_songs_array);		/* these are dynamic arrays */
		safe_free(played_songs_array);
		return;
	    }
	    if (prompt) {
		clear();
		attrset(A_NORMAL);
		mvprintw(0, 0, "Playing %s in %d seconds, press any key to abort...",
			 p->path, number_option_value(OPT_DELAY_BETWEEN_SONGS));
		refresh();
		halfdelay(number_option_value(OPT_DELAY_BETWEEN_SONGS) * 10);
		ch = getch();
		if (ch != ERR) {
		    safe_free(toggled_songs_array);	/* these are dynamic arrays */
		    safe_free(played_songs_array);
		    return;	/* abort playing of playlist if user hit a key */
		}
	    }
	    prompt = TRUE;	/* from now on the user will be given a chance to abort after each song */

	    clear();
	    refresh();
	    snprintf(cmd, sizeof(cmd), "%s \"%s\"", player, p->path);
	    endwin();		/* leave curses */
	    system(cmd);
	    refresh();		/* enter curses again */
	}

	if (continuous)
	    goto while_loop_again;

	safe_free(toggled_songs_array);		/* these are dynamic arrays */
	safe_free(played_songs_array);


    } else {			/* not random */

	/* go through all the entries to see if any file types are not registered */
	for (p = first_entry; p; p = p->next) {
	    /* the function prints error msg if needed */
	    if (!has_registered_filename_extension(p->song)) {
		MSG_FILENAME_EXTENSION_NOT_REGISTERED(p->song);
		return;
	    }
	}

    for_loop_again:

	for (p = first_entry; p; p = p->next) {
	    if (p->toggled) {

		if (extract_player_for_file(p->song, player, sizeof(player)) == NULL) {
		    msg("Player for %s not found in the %s variable.", p->song, option_name(OPT_FILENAME_EXTENSIONS));
		    return;
		}
		if (prompt) {
		    clear();
		    attrset(A_NORMAL);
		    mvprintw(0, 0, "Playing %s in %d seconds, press any key to abort...",
			     p->path, number_option_value(OPT_DELAY_BETWEEN_SONGS));
		    refresh();
		    halfdelay(number_option_value(OPT_DELAY_BETWEEN_SONGS) * 10);
		    ch = getch();
		    if (ch != ERR)
			return;	/* abort playing of playlist if user hit a key */
		}
		prompt = TRUE;	/* from now on the user will be given a chance to abort after each song */

		clear();
		refresh();

		snprintf(cmd, sizeof(cmd), "%s \"%s\"", player, p->path);
		endwin();	/* leave curses temporary */
		system(cmd);
		refresh();	/* enter curses again */
	    }
	}

	if (continuous)
	    goto for_loop_again;

    }
}
